Texture2D colorTexture;
Texture2D normalTexture;
Texture2D positionTexture;
SamplerState pointSampler;



cbuffer Light
{
	float3 lightDir;		// Global direction light
	float ambientLight;		// Gray background light
	float3 dirLightColor;
}

// This could be any BRDF
float3 lightning( float3 _vN, float3 _vL, float3 lightColor, float3 materialColor )
{
	// Gamma-corrected two-sided N dot L light
	float fNdotL = dot( _vN, _vL );
	float fDiffuse = abs( fNdotL * 0.7 + 0.3);
	fDiffuse = pow(fDiffuse, 1.5);

	return lightColor * materialColor * fDiffuse + ambientLight * materialColor;
}


float4 PS( float2 texCoord: TEXCOORD0 ) : SV_TARGET
{
	float3 n = normalize(normalTexture.Sample(pointSampler, texCoord));

	float3 diffuseTex = colorTexture.Sample(pointSampler, texCoord);
	
	// Global direction light
	float3 color = lightning(n, lightDir, dirLightColor, diffuseTex);
	
	return float4(color, 1);
}

technique basic
{
	pass p0
	{
		PixelShader = compile ps_4_0 PS();
	}
}
