Texture2D diffuseTexture;		// Do not change this name: Model.cs is using it
SamplerState diffuseSampler;


struct VS_IN
{
	float3 pos : POSITION;
	float3 normal : NORMAL;
	float2 uv : TEXCOORD;
};

struct PS_IN
{
	float4 pos : SV_POSITION;
	float2 uv : TEXCOORD0;
	float3 normal : TEXCOORD1;
	float3 worldPos : TEXCOORD2;
};

struct PS_OUT
{
	float4 color : SV_TARGET0;
	float4 normals : SV_TARGET1;
	float4 position : SV_TARGET2;
};


cbuffer Transforms
{
	row_major float4x4 worldViewProj;
	row_major float4x4 world;
	row_major float4x4 worldInvTranspose;
	float3 cameraPos;
}

PS_IN VS( VS_IN input )
{
	PS_IN output;
	output.pos = mul(float4(input.pos,1), worldViewProj);
	output.normal = mul(input.normal, (float3x3)worldInvTranspose);
	output.worldPos =  mul(float4(input.pos, 1), world).xyz;
	output.uv = input.uv;
	
	return output;
}

PS_OUT PS( PS_IN input )
{
	PS_OUT output;
	// Fill in the 3 attributes
	output.color.xyz =  diffuseTexture.Sample(diffuseSampler, input.uv).xyz;
	output.normals.xyz = normalize(input.normal);
	output.position.xyz = input.worldPos;

	// Initialize unused components: This is very suboptimale. A
	// good deferred renderer tries to use every bit!
	output.color.w = 1;
	output.normals.w = 1;
	output.position.w = 1;
	
	return output;
}

technique basic
{
	pass p0
	{
		VertexShader = compile vs_4_0 VS();
		PixelShader = compile ps_4_0 PS();
	}
}
