Texture2D colorTexture;
Texture2D normalTexture;
Texture2D positionTexture;
SamplerState pointSampler;



struct PointLight {
	float3 pos;
	float att1;				// -1 / (outer_range-inner_range)
	float3 color;
	float att2;				// 1 + inner_range / (outer_range-inner_range)
	// The inner_range is the distance at which the light starts to fade out.
	// The outer_range is the distance where the light gets 0.
};

cbuffer Lights
{
	PointLight light;
}


float smootherstep(float x) { return (x*x*x)*(x*(x*6.0f - 15.0f) + 10.0f); }

// This could be any BRDF
float3 lightning( float3 _vN, float3 _vL, float3 lightColor, float3 materialColor )
{
	// Gamma-corrected two-sided N dot L light
	float fNdotL = dot( _vN, _vL );
	float fDiffuse = abs( fNdotL * 0.7 + 0.3);
	fDiffuse = pow(fDiffuse, 1.5);

	return lightColor * materialColor * fDiffuse;
}

float4 PS( float2 texCoord: TEXCOORD0 ) : SV_TARGET
{
	// Sample geometry information
	float3 n = normalize(normalTexture.Sample(pointSampler, texCoord)).xyz;
	float3 diffuseTex = colorTexture.Sample(pointSampler, texCoord).xyz;
	float3 worldPos = positionTexture.Sample(pointSampler, texCoord).xyz;
	
	// Calculate the one point-light
	float3 lightDir = light.pos - worldPos;
	float distance = length(lightDir);
	float attentuation = smootherstep( saturate(light.att1 * distance + light.att2) );
	if( attentuation > 0.0 )
	{
		// Normalize - direction is really required
		lightDir /= distance;
		return float4(lightning( n, lightDir, light.color, diffuseTex ) * attentuation, 1);
	}

	return float4(0,0,0,0);
}

technique basic
{
	pass p0
	{
		PixelShader = compile ps_4_0 PS();
	}
}
